@@ -3,7 +3,7 @@
  * \brief Properties class definition
  * \see mlt_properties_s
  *
- * Copyright (C) 2003-2020 Meltytech, LLC
+ * Copyright (C) 2003-2021 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -44,6 +44,8 @@
 #include <locale.h>
 #include <float.h>
 
+#define MAX_LOAD_LINE_SIZE 4096
+
 /** \brief private implementation of the property list */
 
 typedef struct
@@ -57,6 +59,9 @@ typedef struct
 	int ref_count;
 	pthread_mutex_t mutex;
 	locale_t locale;
+	mlt_properties* children_properties;
+	char** children_names;
+	int children_count;
 }
 property_list;
 
@@ -206,11 +211,11 @@ static int load_properties( mlt_properties self, const char *filename )
 	if ( file != NULL )
 	{
 		// Temp string
-		char temp[ 1024 ];
-		char last[ 1024 ] = "";
+		char temp[ MAX_LOAD_LINE_SIZE ];
+		char last[ MAX_LOAD_LINE_SIZE ] = "";
 
 		// Read each string from the file
-		while( fgets( temp, 1024, file ) )
+		while( fgets( temp, MAX_LOAD_LINE_SIZE, file ) )
 		{
 			// Chomp the new line character from the string
 			int x = strlen( temp ) - 1;
@@ -220,7 +225,7 @@ static int load_properties( mlt_properties self, const char *filename )
 			// Check if the line starts with a .
 			if ( temp[ 0 ] == '.' )
 			{
-				char temp2[ 1024 ];
+				char temp2[ MAX_LOAD_LINE_SIZE ];
 				strcpy( temp2, last );
 				strncat( temp2, temp, sizeof(temp2) - strlen(temp2) - 1 );
 				strcpy( temp, temp2 );
@@ -456,12 +461,24 @@ int mlt_properties_inherit( mlt_properties self, mlt_properties that )
 	int i = 0;
 	for ( i = 0; i < count; i ++ )
 	{
-		char *value = mlt_properties_get_value( that, i );
-		if ( value != NULL )
+		char *name = mlt_properties_get_name( that, i );
+		if (name && strcmp("properties", name))
 		{
-			char *name = mlt_properties_get_name( that, i );
-			if (name && strcmp("properties", name))
+			char *value = mlt_properties_get_value( that, i );
+			if ( value != NULL )
+			{
 				mlt_properties_set_string( self, name, value );
+			}
+			else
+			{
+				mlt_properties that_child_props = mlt_properties_get_properties_at( that, i );
+				if ( that_child_props != NULL )
+				{
+					mlt_properties child_props = mlt_properties_new();
+					mlt_properties_set_properties( self, name, child_props );
+					mlt_properties_inherit( child_props, that_child_props );
+				}
+			}
 		}
 	}
 
@@ -597,6 +614,11 @@ static mlt_property mlt_properties_fetch( mlt_properties self, const char *name
 	return property;
 }
 
+static void fire_property_changed(mlt_properties self, const char *name)
+{
+	mlt_events_fire(self, "property-changed", mlt_event_data_from_string(name));
+}
+
 /** Copy a property to another properties list.
  *
  * \public \memberof mlt_properties_s
@@ -614,7 +636,7 @@ void mlt_properties_pass_property( mlt_properties self, mlt_properties that, con
 		return;
 
 	mlt_property_pass( mlt_properties_fetch( self, name ), that_prop );
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 }
 
 /** Copy all properties specified in a comma-separated list to another properties list.
@@ -786,7 +808,7 @@ int mlt_properties_set( mlt_properties self, const char *name, const char *value
 			mlt_properties_preset( self, value );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -848,7 +870,7 @@ int mlt_properties_set_string( mlt_properties self, const char *name, const char
 			mlt_properties_preset( self, value );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -1051,7 +1073,7 @@ int mlt_properties_set_int( mlt_properties self, const char *name, int value )
 		mlt_properties_do_mirror( self, name );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -1095,7 +1117,7 @@ int mlt_properties_set_int64( mlt_properties self, const char *name, int64_t val
 		mlt_properties_do_mirror( self, name );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -1147,7 +1169,7 @@ int mlt_properties_set_double( mlt_properties self, const char *name, double val
 		mlt_properties_do_mirror( self, name );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -1199,7 +1221,7 @@ int mlt_properties_set_position( mlt_properties self, const char *name, mlt_posi
 		mlt_properties_do_mirror( self, name );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -1245,7 +1267,7 @@ int mlt_properties_set_data( mlt_properties self, const char *name, void *value,
 	if ( property != NULL )
 		error = mlt_property_set_data( property, value, length, destroy, serialise );
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -1324,8 +1346,10 @@ void mlt_properties_debug( mlt_properties self, const char *title, FILE *output
 		for ( i = 0; i < list->count; i ++ )
 			if ( mlt_properties_get( self, list->name[ i ] ) != NULL )
 				fprintf( output, ", %s=%s", list->name[ i ], mlt_properties_get( self, list->name[ i ] ) );
-			else
+			else if ( mlt_properties_get_data( self, list->name[ i ], NULL ) != NULL )
 				fprintf( output, ", %s=%p", list->name[ i ], mlt_properties_get_data( self, list->name[ i ], NULL ) );
+			else
+				fprintf( output, ", %s=%p", list->name[ i ], mlt_properties_get_properties( self, list->name[ i ] ) );
 		fprintf( output, " ]" );
 	}
 	fprintf( output, "\n" );
@@ -1870,7 +1894,7 @@ mlt_properties mlt_properties_parse_yaml( const char *filename )
 		if ( file )
 		{
 			// Temp string
-			char temp[ 1024 ];
+			char temp[ MAX_LOAD_LINE_SIZE ];
 			char *ptemp = &temp[ 0 ];
 
 			// Default to LC_NUMERIC = C
@@ -1884,7 +1908,7 @@ mlt_properties mlt_properties_parse_yaml( const char *filename )
 			mlt_deque_push_back_int( context->index_stack, 0 );
 
 			// Read each string from the file
-			while( fgets( temp, 1024, file ) )
+			while( fgets( temp, MAX_LOAD_LINE_SIZE, file ) )
 			{
 				// Check for end-of-stream
 				if ( strncmp( ptemp, "...", 3 ) == 0 )
@@ -2244,7 +2268,7 @@ void mlt_properties_clear( mlt_properties self, const char *name )
 	if ( property )
 		mlt_property_clear( property );
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 }
 
 /** Check if a property exists.
@@ -2405,7 +2429,7 @@ int mlt_properties_set_color( mlt_properties self, const char *name, mlt_color c
 		mlt_properties_do_mirror( self, name );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -2467,7 +2491,7 @@ int mlt_properties_anim_set( mlt_properties self, const char *name, const char *
 		mlt_properties_do_mirror( self, name );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -2525,7 +2549,7 @@ int mlt_properties_anim_set_int( mlt_properties self, const char *name, int valu
 		mlt_properties_do_mirror( self, name );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -2583,7 +2607,7 @@ int mlt_properties_anim_set_double( mlt_properties self, const char *name, doubl
 		mlt_properties_do_mirror( self, name );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -2627,7 +2651,7 @@ extern int mlt_properties_set_rect( mlt_properties self, const char *name, mlt_r
 		mlt_properties_do_mirror( self, name );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -2681,7 +2705,7 @@ extern int mlt_properties_anim_set_rect( mlt_properties self, const char *name,
 		mlt_properties_do_mirror( self, name );
 	}
 
-	mlt_events_fire( self, "property-changed", name, NULL );
+	fire_property_changed(self, name);
 
 	return error;
 }
@@ -2733,3 +2757,40 @@ int mlt_properties_from_utf8( mlt_properties properties, const char *name_from,
 }
 
 #endif
+
+int mlt_properties_set_properties( mlt_properties self, const char *name, mlt_properties properties )
+{
+	int error = 1;
+
+	if ( !self || !name || !properties) return error;
+
+	// Fetch the property to work with
+	mlt_property property = mlt_properties_fetch( self, name );
+
+	// Set it if not NULL
+	if ( property != NULL )
+	{
+		error = mlt_property_set_properties( property, properties );
+		mlt_properties_do_mirror( self, name );
+	}
+
+	fire_property_changed(self, name);
+
+	return error;
+}
+
+mlt_properties mlt_properties_get_properties( mlt_properties self, const char *name )
+{
+	property_list *list = self->local;
+	mlt_property value = mlt_properties_find( self, name );
+	return value == NULL ? NULL : mlt_property_get_properties( value );
+}
+
+mlt_properties mlt_properties_get_properties_at( mlt_properties self, int index )
+{
+	if ( !self ) return NULL;
+	property_list *list = self->local;
+	if ( index >= 0 && index < list->count )
+		return mlt_property_get_properties( list->value[ index ] );
+	return NULL;
+}
