@@ -34,6 +34,7 @@ extern mlt_filter filter_avfilter_init( mlt_profile, mlt_service_type, const cha
 
 // ffmpeg Header files
 #include <libavformat/avformat.h>
+#include <libavcodec/avcodec.h>
 #ifdef AVDEVICE
 #include <libavdevice/avdevice.h>
 #endif
@@ -46,55 +47,14 @@ extern mlt_filter filter_avfilter_init( mlt_profile, mlt_service_type, const cha
 // A static flag used to determine if avformat has been initialised
 static int avformat_initialised = 0;
 
-static int avformat_lockmgr(void **mutex, enum AVLockOp op)
-{
-	pthread_mutex_t** pmutex = (pthread_mutex_t**) mutex;
-
-	switch (op)
-	{
-	case AV_LOCK_CREATE:
-		*pmutex = (pthread_mutex_t*) malloc(sizeof(pthread_mutex_t));
-		if (!*pmutex) return -1;
-		pthread_mutex_init(*pmutex, NULL);
-		break;
-	case AV_LOCK_OBTAIN:
-		if (!*pmutex) return -1;
-		pthread_mutex_lock(*pmutex);
-		break;
-	case AV_LOCK_RELEASE:
-		if (!*pmutex) return -1;
-		pthread_mutex_unlock(*pmutex);
-		break;
-	case AV_LOCK_DESTROY:
-		if (!*pmutex) return -1;
-		pthread_mutex_destroy(*pmutex);
-		free(*pmutex);
-		*pmutex = NULL;
-		break;
-	}
-
-	return 0;
-}
-
-static void unregister_lockmgr( void *p )
-{
-	av_lockmgr_register( NULL );
-}
-
 static void avformat_init( )
 {
 	// Initialise avformat if necessary
 	if ( avformat_initialised == 0 )
 	{
 		avformat_initialised = 1;
-		av_lockmgr_register( &avformat_lockmgr );
-		mlt_factory_register_for_clean_up( &avformat_lockmgr, unregister_lockmgr );
-		av_register_all( );
 #ifdef AVDEVICE
 		avdevice_register_all();
-#endif
-#ifdef AVFILTER
-		avfilter_register_all();
 #endif
 		avformat_network_init();
 		av_log_set_level( mlt_log_get_level() );
@@ -112,9 +72,9 @@ static void *create_service( mlt_profile profile, mlt_service_type type, const c
 #ifdef CODECS
 	if ( !strncmp( id, "avformat", 8 ) )
 	{
-		if ( type == producer_type )
+		if ( type == mlt_service_producer_type )
 			return producer_avformat_init( profile, id, arg );
-		else if ( type == consumer_type )
+		else if ( type == mlt_service_consumer_type )
 			return consumer_avformat_init( profile, arg );
 	}
 #endif
@@ -294,52 +254,61 @@ static mlt_properties avformat_metadata( mlt_service_type type, const char *id,
 	// Convert the service type to a string.
 	switch ( type )
 	{
-		case consumer_type:
+		case mlt_service_consumer_type:
 			service_type = "consumer";
 			break;
-		case filter_type:
+		case mlt_service_filter_type:
 			service_type = "filter";
 			break;
-		case producer_type:
+		case mlt_service_producer_type:
 			service_type = "producer";
 			break;
-		case transition_type:
+		case mlt_service_transition_type:
 			service_type = "transition";
 			break;
 		default:
 			return NULL;
 	}
+
+	if ( type == mlt_service_producer_type && !strcmp( id, "avformat-novalidate" ) )
+	{
+		id = "avformat";
+	}
+
 	// Load the yaml file
 	snprintf( file, PATH_MAX, "%s/avformat/%s_%s.yml", mlt_environment( "MLT_DATA" ), service_type, id );
 	result = mlt_properties_parse_yaml( file );
-	if ( result && ( type == consumer_type || type == producer_type ) )
+	if ( result && ( type == mlt_service_consumer_type || type == mlt_service_producer_type ) )
 	{
 		// Annotate the yaml properties with AVOptions.
 		mlt_properties params = (mlt_properties) mlt_properties_get_data( result, "parameters", NULL );
 		AVFormatContext *avformat = avformat_alloc_context();
 		AVCodecContext *avcodec = avcodec_alloc_context3( NULL );
-		int flags = ( type == consumer_type )? AV_OPT_FLAG_ENCODING_PARAM : AV_OPT_FLAG_DECODING_PARAM;
+		int flags = ( type == mlt_service_consumer_type )? AV_OPT_FLAG_ENCODING_PARAM : AV_OPT_FLAG_DECODING_PARAM;
 
 		add_parameters( params, avformat, flags, NULL, NULL, NULL );
 		avformat_init();
-		if ( type == producer_type )
+		if ( type == mlt_service_producer_type )
 		{
-			AVInputFormat *f = NULL;
-			while ( ( f = av_iformat_next( f ) ) )
+			const AVInputFormat *f = NULL;
+			void *iterator = NULL;
+			while ( ( f = av_demuxer_iterate( &iterator ) ) )
 				if ( f->priv_class )
 					add_parameters( params, &f->priv_class, flags, NULL, f->name, NULL );
 		}
 		else
 		{
-			AVOutputFormat *f = NULL;
-			while ( ( f = av_oformat_next( f ) ) )
+			const AVOutputFormat *f = NULL;
+			void *iterator = NULL;
+			while ( ( f = av_muxer_iterate( &iterator ) ) )
 				if ( f->priv_class )
 					add_parameters( params, &f->priv_class, flags, NULL, f->name, NULL );
 		}
 
 		add_parameters( params, avcodec, flags, NULL, NULL, NULL );
-		AVCodec *c = NULL;
-		while ( ( c = av_codec_next( c ) ) )
+		const AVCodec *c = NULL;
+		void *iterator = NULL;
+		while ( ( c = av_codec_iterate( &iterator ) ) )
 			if ( c->priv_class )
 				add_parameters( params, &c->priv_class, flags, NULL, c->name, NULL );
 
@@ -420,17 +389,17 @@ static mlt_properties avfilter_metadata( mlt_service_type type, const char *id,
 MLT_REPOSITORY
 {
 #ifdef CODECS
-	MLT_REGISTER( consumer_type, "avformat", create_service );
-	MLT_REGISTER( producer_type, "avformat", create_service );
-	MLT_REGISTER( producer_type, "avformat-novalidate", create_service );
-	MLT_REGISTER_METADATA( consumer_type, "avformat", avformat_metadata, NULL );
-	MLT_REGISTER_METADATA( producer_type, "avformat", avformat_metadata, NULL );
+	MLT_REGISTER( mlt_service_consumer_type, "avformat", create_service );
+	MLT_REGISTER( mlt_service_producer_type, "avformat", create_service );
+	MLT_REGISTER( mlt_service_producer_type, "avformat-novalidate", create_service );
+	MLT_REGISTER_METADATA( mlt_service_consumer_type, "avformat", avformat_metadata, NULL );
+	MLT_REGISTER_METADATA( mlt_service_producer_type, "avformat", avformat_metadata, NULL );
 #endif
 #ifdef FILTERS
-	MLT_REGISTER( filter_type, "avcolour_space", create_service );
-	MLT_REGISTER( filter_type, "avcolor_space", create_service );
-	MLT_REGISTER( filter_type, "avdeinterlace", create_service );
-	MLT_REGISTER( filter_type, "swscale", create_service );
+	MLT_REGISTER( mlt_service_filter_type, "avcolour_space", create_service );
+	MLT_REGISTER( mlt_service_filter_type, "avcolor_space", create_service );
+	MLT_REGISTER( mlt_service_filter_type, "avdeinterlace", create_service );
+	MLT_REGISTER( mlt_service_filter_type, "swscale", create_service );
 
 #ifdef AVFILTER
 	char dirname[PATH_MAX];
@@ -446,9 +415,9 @@ MLT_REPOSITORY
 	mlt_properties_set_data(mlt_global_properties(), "avfilter.resolution_scale",
 		mlt_properties_parse_yaml(dirname), 0, (mlt_destructor) mlt_properties_close, NULL);
 
-	avfilter_register_all();
-	AVFilter *f = NULL;
-	while ( ( f = (AVFilter*)avfilter_next( f ) ) ) {
+	const AVFilter *f = NULL;
+	void *iterator = NULL;
+	while ( ( f = (AVFilter*) av_filter_iterate( &iterator ) ) ) {
 		// Support filters that have one input and one output of the same type.
 		if ( avfilter_pad_count( f->inputs ) == 1 &&
 			avfilter_pad_count( f->outputs ) == 1 &&
@@ -457,14 +426,14 @@ MLT_REPOSITORY
 		{
 			char service_name[1024]="avfilter.";
 			strncat( service_name, f->name, sizeof( service_name ) - strlen( service_name ) -1 );
-			MLT_REGISTER( filter_type, service_name, filter_avfilter_init );
-			MLT_REGISTER_METADATA( filter_type, service_name, avfilter_metadata, (void*)f->name );
+			MLT_REGISTER( mlt_service_filter_type, service_name, filter_avfilter_init );
+			MLT_REGISTER_METADATA( mlt_service_filter_type, service_name, avfilter_metadata, (void*)f->name );
 		}
 	}
 	mlt_properties_close( blacklist );
 #endif // AVFILTER
 #endif
 #ifdef SWRESAMPLE
-	MLT_REGISTER( filter_type, "swresample", create_service );
+	MLT_REGISTER( mlt_service_filter_type, "swresample", create_service );
 #endif
 }
